package sun.misc;

import java.util.Enumeration;

public class Queue<T>
{
    int length;
    QueueElement<T> head;
    QueueElement<T> tail;
    
    public Queue() {
        this.length = 0;
        this.head = null;
        this.tail = null;
    }
    
    public synchronized void enqueue(T t) {
        QueueElement<T> queueElement = new QueueElement(t);
        if (this.head == null) {
            this.head = queueElement;
            this.tail = queueElement;
            this.length = 1;
        }
        else {
            queueElement.next = this.head;
            this.head.prev = queueElement;
            this.head = queueElement;
            ++this.length;
        }
        this.notify();
    }
    
    public T dequeue() throws InterruptedException {
        return this.dequeue(0L);
    }
    
    public synchronized T dequeue(long timeoutMillis) throws InterruptedException {
        while (this.tail == null) {
            this.wait(timeoutMillis);
        }
        QueueElement<Object> tail = this.tail;
        this.tail = tail.prev;
        if (this.tail == null) {
            this.head = null;
        }
        else {
            this.tail.next = null;
        }
        --this.length;
        return tail.obj;
    }
    
    public synchronized boolean isEmpty() {
        return this.tail == null;
    }
    
    public final synchronized Enumeration<T> elements() {
        return new LIFOQueueEnumerator(this);
    }
    
    public final synchronized Enumeration<T> reverseElements() {
        return new FIFOQueueEnumerator(this);
    }
    
    public synchronized void dump(String str) {
        System.err.println(">> " + str);
        System.err.println("[" + this.length + " elt(s); head = " + ((this.head == null) ? "null" : (this.head.obj + "")) + " tail = " + ((this.tail == null) ? "null" : (this.tail.obj + "")));
        QueueElement<Object> obj = this.head;
        QueueElement<Object> obj2 = null;
        while (obj != null) {
            System.err.println("  " + obj);
            obj2 = obj;
            obj = obj.next;
        }
        if (obj2 != this.tail) {
            System.err.println("  tail != last: " + this.tail + ", " + obj2);
        }
        System.err.println("]");
    }
}
